#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <be/app/Roster.h>
#include <be/app/Application.h>

#include <TrackerAddOn.h>

#include <storage/FindDirectory.h>
#include <storage/Directory.h>
#include <storage/Path.h>
#include <storage/Node.h>
#include <storage/NodeInfo.h>
#include <kernel/fs_attr.h>

#include <support/String.h>

#include <interface/InterfaceDefs.h>
#include <interface/GraphicsDefs.h>
#include <interface/PopUpMenu.h>
#include <interface/MenuItem.h>
#include <interface/Alert.h>
#include <interface/Screen.h>
#include <interface/Bitmap.h>

#include "mouseWin.h"
#include "setWin.h"
#include "Colors.h"
#include "IconMenuItem.h"
#include "Label.h"

void		AddLabel			(BMessage *msg, BString * label, bool DoTint, bool DoModify, BMessage *settings);
void		RemoveLabel			(BMessage *msg, bool DoModify);
void		tint_icon_data		(uchar *icondata, int size, rgb_color tint);
void		color_icon			(uchar *icondata, int size, rgb_color color);
status_t 	MyGetTrackerIcon	(entry_ref* ref, BBitmap* large_icon, BBitmap* mini_icon);
void		error_check			(const char * message, int32 status);

extern "C" void 
process_refs(entry_ref dir_ref, BMessage *msg, void *)
{
	be_roster->Launch ("application/x-vnd.kirilla.label", msg);
}

int main() 
{
	Label * app	=	new Label ();
	app->Run();
	return (0);
}

void error_check(const char * message, int32 status)
{
	printf(message);
	printf("Explanation: %s\n", strerror( status ));
}

void 
tint_icon_data(uchar *icondata, int size, rgb_color tint) 
{ 
    for (int i=0;  i<size;  i++)
   { 
		int cval = icondata[i];
		if (cval != 255)	/* don't convert transparent pixels */
		{
				BScreen screen;	// needed for IndexForColor()
				rgb_color current = system_colors()->color_list[cval];
				cval = screen.IndexForColor(
						(int)current.red	*0.50 + tint.red	*0.50,
						(int)current.green	*0.50 + tint.green	*0.50, 
						(int)current.blue	*0.50 + tint.blue	*0.50);
		}
		icondata[i] = cval;
	}
}

void 
color_icon(uchar *icondata, int size, rgb_color color) 
{ 
    for (int i=0;  i<size;  i++)
   { 
		int cval = icondata[i];
		if (cval != 255)	/* don't convert transparent pixels */
		{
				BScreen screen;	// needed for IndexForColor()
				rgb_color current = system_colors()->color_list[cval];
				cval = screen.IndexForColor(current.red		= color.red,
											current.green	= color.green, 
											current.blue	= color.blue);
		}
		icondata[i] = cval;
	}
}

void
AddLabel	(BMessage *msg, BString *label, bool DoTint, bool DoModify, BMessage *settings) 
{ 
	rgb_color	color;
	if (*label == settings->FindString("label1", (int32) 0))	color = Orange;
	if (*label == settings->FindString("label2", (int32) 0))	color = Red;
	if (*label == settings->FindString("label3", (int32) 0))	color = Lavender;
	if (*label == settings->FindString("label4", (int32) 0))	color = LightBlue;
	if (*label == settings->FindString("label5", (int32) 0))	color = Blue;
	if (*label == settings->FindString("label6", (int32) 0))	color = Green;
	if (*label == settings->FindString("label7", (int32) 0))	color = Black;

	BBitmap	* 	large_icon	= NULL;
	BBitmap *	mini_icon	= NULL;

	status_t	status	= 	B_OK;
	entry_ref	ref;
	while (msg->FindRef("refs", &ref) == B_OK)
	{
		BEntry		entry		(&ref, true);
		BNode 		node		(&entry);
		BNodeInfo	node_info	(&node);
		attr_info	a_info;
		
		if (large_icon != NULL)	delete large_icon;
		if (mini_icon != NULL)	delete mini_icon;
		
		large_icon	=	new BBitmap (BRect(0,0,31,31), B_CMAP8, true);
		mini_icon	=	new BBitmap (BRect(0,0,15,15), B_CMAP8, true);
		
		if ((status = node.GetAttrInfo("META:label", &a_info)) == B_OK)	// Label
		{
			printf("LABEL\n");
			
			if ((status = node.WriteAttr("META:label", B_STRING_TYPE, 0, label->String(), label->Length()+1)) <= 0)
			if (status <= 0) error_check("node.WriteAttr(META:label) ", status);			//	Add Label
			
			if (DoTint)
			{
				if ((status = node.GetAttrInfo("LABEL:L:ICON_BKP", &a_info)) != B_OK)	// No Backup
				{
					printf("NO BACKUP-ICON\n");		// hmm.. En NO-ICON -fil, som vi inte kan göra GetTrackerIcon på
													// eftersom vi inte vill ackumulera tints
	
					if ((status= MyGetTrackerIcon(&ref, large_icon, mini_icon)) == B_OK)
					{
						tint_icon_data((uchar*) large_icon->Bits(), large_icon->BitsLength(), color);		
						tint_icon_data((uchar*) mini_icon->Bits(), mini_icon->BitsLength(), color);
					
						if ((status = node_info.SetIcon(large_icon, B_LARGE_ICON)) != B_OK)			
							error_check("node_info.SetIcon(large_icon) ", status);
						if ((status = node_info.SetIcon(mini_icon, B_MINI_ICON)) != B_OK)
							error_check("node_info.SetIcon(mini_icon) ", status);
					}
					else	error_check("MyGetTrackerIcon() ", status);
				}
				else																	// Backup
				{	
					printf("GOT BACKUP-ICON\n");
																							// Restore Icon
					char * buf = new char [a_info.size +1];
					if ((status = node.ReadAttr("LABEL:L:ICON_BKP", 'ICON', 0, (void*) buf, a_info.size)) > 0)
					{	if ((status = node.WriteAttr("BEOS:L:STD_ICON", 'ICON', 0, (void*) buf, a_info.size)) <= 0)
							error_check("node.WriteAttr(BEOS:L:STD_ICON) ", status); }
					else error_check("node.ReadAttr(LABEL:L:ICON_BKP) ", status);
					delete buf;
					
					if ((status = node.GetAttrInfo("LABEL:M:ICON_BKP", &a_info)) == B_OK)
					{	buf = new char [a_info.size +1];
						if ((status = node.ReadAttr("LABEL:M:ICON_BKP", 'MICN', 0, (void*) buf, a_info.size)) > 0)
						{	if ((status = node.WriteAttr("BEOS:M:STD_ICON", 'MICN', 0, (void*) buf, a_info.size)) <= 0)
								error_check("node.WriteAttr(BEOS:M:STD_ICON) ", status);	}
						else error_check("node.ReadAttr(LABEL:M:ICON_BKP) ", status);
						delete buf;	}
					//
					if ((status = node_info.GetTrackerIcon(large_icon, B_LARGE_ICON)) != B_OK)	// GetTrackerIcon
						error_check("node_info.GetTrackerIcon(large_icon) ", status);			// Tint Icon
					if ((status = node_info.GetTrackerIcon(mini_icon, B_MINI_ICON)) != B_OK)	// Set Icon
						error_check("node_info.GetTrackerIcon(mini_icon) ", status);
					
					tint_icon_data((uchar*) large_icon->Bits(), large_icon->BitsLength(), color);		
					tint_icon_data((uchar*) mini_icon->Bits(), mini_icon->BitsLength(), color);
					
					if ((status = node_info.SetIcon(large_icon, B_LARGE_ICON)) != B_OK)			
						error_check("node_info.SetIcon(large_icon) ", status);
					if ((status = node_info.SetIcon(mini_icon, B_MINI_ICON)) != B_OK)
						error_check("node_info.SetIcon(mini_icon) ", status);
				}
			}
		}
		else																		// No Label
		{
			printf("NO LABEL\n");

			if ((status = node.WriteAttr("META:label", B_STRING_TYPE, 0, label->String(), label->Length()+1)) <= 0)
			if (status <= 0) error_check("node.WriteAttr(META:label) ", status);			//	Add Label
			
			if ((status = node.GetAttrInfo("BEOS:L:STD_ICON", &a_info)) != B_OK)	// No Icon
			{
				printf("NO ICON\n");
				if (DoTint)
				{
					if((status = MyGetTrackerIcon(&ref, large_icon, mini_icon)) == B_OK)
					{
						tint_icon_data((uchar*) large_icon->Bits(), large_icon->BitsLength(), color);		
						tint_icon_data((uchar*) mini_icon->Bits(), mini_icon->BitsLength(), color);
					
						if ((status = node_info.SetIcon(large_icon, B_LARGE_ICON)) != B_OK)			
							error_check("node_info.SetIcon(large_icon) ", status);
						if ((status = node_info.SetIcon(mini_icon, B_MINI_ICON)) != B_OK)
							error_check("node_info.SetIcon(mini_icon) ", status);
					}
					else	error_check("MyGetTrackerIcon() ", status);
				}
			}
			else																	// Icon
			{
				printf("GOT ICON\n");
				
				char * buf = new char [a_info.size +1];										// Backup Icon
				if ((status = node.ReadAttr("BEOS:L:STD_ICON", 'ICON', 0, (void*) buf, a_info.size)) > 0)
				{	if ((status = node.WriteAttr("LABEL:L:ICON_BKP", 'ICON', 0, (void*) buf, a_info.size)) <= 0)
						error_check("node.WriteAttr(LABEL:L:ICON_BKP) ", status);	}
				else error_check("node.ReadAttr(BEOS:L:STD_ICON) ", status);
				delete buf;
				
				if ((status = node.GetAttrInfo("BEOS:M:STD_ICON", &a_info)) == B_OK)
				{	buf = new char [a_info.size +1];
					if ((status = node.ReadAttr("BEOS:M:STD_ICON", 'MICN', 0, (void*) buf, a_info.size)) > 0)
					{	if ((status = node.WriteAttr("LABEL:M:ICON_BKP", 'MICN', 0, (void*) buf, a_info.size)) <= 0)
							error_check("node.WriteAttr(LABEL:M:ICON_BKP) ", status);	}
					else error_check("node.ReadAttr(BEOS:M:STD_ICON) ", status);
					delete buf;	}
				
				if (DoTint)
				{ 
					if ((status = node_info.GetTrackerIcon(large_icon, B_LARGE_ICON)) != B_OK)	// GetTrackerIcon
						error_check("node_info.GetTrackerIcon(large_icon) ", status);			// Tint Icon
					if ((status = node_info.GetTrackerIcon(mini_icon, B_MINI_ICON)) != B_OK)	// Set Icon
						error_check("node_info.GetTrackerIcon(mini_icon) ", status);
					
					tint_icon_data((uchar*) large_icon->Bits(), large_icon->BitsLength(), color);		
					tint_icon_data((uchar*) mini_icon->Bits(), mini_icon->BitsLength(), color);
					
					if ((status = node_info.SetIcon(large_icon, B_LARGE_ICON)) != B_OK)			
						error_check("node_info.SetIcon(large_icon) ", status);
					if ((status = node_info.SetIcon(mini_icon, B_MINI_ICON)) != B_OK)
						error_check("node_info.SetIcon(mini_icon) ", status);
				}
			}
		}
			
		if (DoModify)	//	touch MODIFIED
		{
			BEntry	entry(&ref, true);
			entry.	SetModificationTime(time(NULL));
		}

		msg->RemoveData("refs");
	}
}

status_t 
MyGetTrackerIcon	(entry_ref* ref, BBitmap* large_icon, BBitmap* mini_icon)
{ 
	status_t	status	= 	B_OK;
	status_t	status2	=	B_OK;

	BEntry		entry		(ref, true);
	BNode 		node		(&entry);
	BNodeInfo	node_info	(&node);
		
	if (large_icon	==	NULL)	large_icon	=	new BBitmap (BRect(0,0,31,31), B_CMAP8, true);
	if (mini_icon	==	NULL)	mini_icon	=	new BBitmap (BRect(0,0,15,15), B_CMAP8, true);
		
	bool GotIcon = false;
	char * type	= new char[B_MIME_TYPE_LENGTH];
	node_info.GetType(type);
	printf("File: %s (%s)\n", ref->name, type);
	
	// 1 - 	Icon from Node's Pref_app.
	char * sig	= new char[B_MIME_TYPE_LENGTH];
	if ((status = node_info.GetPreferredApp(sig)) == B_OK)
	{
		BMimeType node_pref_app(sig);
		
		if ((status = node_pref_app.GetIconForType(type, large_icon, B_LARGE_ICON))	== B_OK &&
			(status2 = node_pref_app.GetIconForType(type, mini_icon, B_MINI_ICON)) == B_OK)
		{
			GotIcon = true;
			printf("GOT ICON: from Node's Pref_app (%s)**\n", sig);
		}
		else
		{
			printf("NO ICON: Node->PrefApp has no icon.\n");
		}
	}
	else	printf("NO ICON: Node has no PrefApp.\n");
	delete	sig;
	
	// 2 -	Icon from Filetype.
	if (! GotIcon)
	{
		BMimeType mime(type);
		if ((status = mime.GetIcon(large_icon, B_LARGE_ICON)) == B_OK &&
			(status2 = mime.GetIcon(mini_icon, B_MINI_ICON)) == B_OK)
		{
			GotIcon = true;
			printf("GOT ICON: from filetype\n");
		}
		else
		{
			printf("NO ICON: Filetype (%s) has no icon.\n", type);
		}
	}

	// 3 -	Icon from Filtype's Pref_app.
	if (! GotIcon)
	{
		BMimeType mime(type);
		char * sig	= new char[B_MIME_TYPE_LENGTH];
		if ((status = mime.GetPreferredApp(sig)) == B_OK)
		{
			BMimeType type_pref_app(sig);
			if ((status = type_pref_app.GetIconForType(type, large_icon, B_LARGE_ICON))	== B_OK &&
				(status2 = type_pref_app.GetIconForType(type, mini_icon, B_MINI_ICON)) == B_OK)
			{
				GotIcon = true;
				printf("GOT ICON: from Filetype's Pref.App. (%s)\n", sig);
			}
			else	
			{
				printf("NO ICON: Filetype's Pref_App (%s) has no icon.\n", sig);
			}
		}
		else	printf("NO ICON: Filetype has no Pref_App.\n");
		delete sig;
	}

	// 4 -	Icon from Supertype.
	if (! GotIcon)
	{
		BString	temp	=	type;
		int32	cut		=	temp.FindFirst('/');
		temp.Truncate(cut);	
		
		BMimeType super(temp.String());
		if ((status = super.GetIcon(large_icon, B_LARGE_ICON)) == B_OK &&
			(status2 = super.GetIcon(mini_icon, B_MINI_ICON)) == B_OK)
		{
				GotIcon = true;
				printf("GOT ICON: Icon from Supertype (%s).\n", temp.String());
		}
		else
		{
			printf("NO ICON: Supertype (%s) does not specify an icon.\n", temp.String());
		}
	}

	// 5 -	Icon from Supertype's Pref_app.
	if (! GotIcon)
	{
		BString	temp	=	type;
		int32	cut		=	temp.FindFirst('/');
		temp.Truncate(cut);	
				
		BMimeType super(temp.String());
		char * sig	= new char[B_MIME_TYPE_LENGTH];
		if ((status = super.GetPreferredApp(sig)) == B_OK)
		{
			BMimeType super_pref_app(sig);
			if ((status = super_pref_app.GetIconForType(type, large_icon, B_LARGE_ICON)) == B_OK &&
				(status2 = super_pref_app.GetIconForType(type, mini_icon, B_MINI_ICON)) == B_OK)
			{
				GotIcon = true;
				printf("GOT ICON: from Supertype's Pref_App (%s).\n", sig);
			}
			else
			{
				printf("NO ICON: Supertype (%s)'s Pref_App (%s) does not specify an icon.\n", temp.String(), sig);
			}
		}
		else	printf("NO ICON: Supertype (%s) does not have a Pref_App.\n", temp.String());
		delete sig;
	}
	
	// 6 -	GetTrackerIcon()
	if (! GotIcon)
	{
		if ((status = node_info.GetTrackerIcon(large_icon, B_LARGE_ICON)) == B_OK &&
			(status2 = node_info.GetTrackerIcon(mini_icon, B_MINI_ICON)) == B_OK)
		{
				GotIcon = true;
				printf("GOT ICON: GetTrackerIcon()\n");
		}
		else	
		{
				printf("NO ICON: GetTrackerIcon()\n");
				error_check("Large: ", status);
				error_check("Mini: ", status2);
		}
	}
	
	delete type;
	if (GotIcon) 	return B_OK;
	else			return B_ERROR;
}

void 
RemoveLabel	(BMessage *msg, bool DoModify)
{
	entry_ref	ref;
	while (msg->FindRef("refs", &ref) == B_OK)
	{
		BEntry		entry		(&ref, true);
		BNode 		node		(&entry);
		attr_info	a_info;
		status_t	status;
		
		if ((status = node.GetAttrInfo("META:label", &a_info)) == B_OK)		// Label
		{
			printf("GOT LABEL\n");
			
			//if ((status = node.WriteAttr("META:label", B_STRING_TYPE, 0, " ", 2)) != B_OK)	// Hack, update Tracker
			//if (status <= 0) error_check("node.WriteAttr(META:label) ", status);			//	Add Label
		
			if ((status = node.RemoveAttr("META:label")) != B_OK)							// Remove label
				error_check("node.RemoveAttr(META:label) ", status);
				
			if ((status = node.GetAttrInfo("LABEL:L:ICON_BKP", &a_info)) != B_OK)	// No Backup
			{
				printf("NO BACKUP-ICON\n");
				
				if ((status = node.RemoveAttr("BEOS:L:STD_ICON")) != B_OK)					// Remove ICON
					error_check("node.RemoveAttr(BEOS:L:STD_ICON) ", status);
				if ((status = node.RemoveAttr("BEOS:M:STD_ICON")) != B_OK)
					error_check("node.RemoveAttr(BEOS:M:STD_ICON) ", status);
				
			}
			else
			{
				printf("GOT BACKUP-ICON\n");										// Backup
				
				char * buf = new char [a_info.size +1];										// Restore ICON
				if ((status = node.ReadAttr("LABEL:L:ICON_BKP", 'ICON', 0, (void*) buf, a_info.size)) > 0)
				{	if ((status = node.WriteAttr("BEOS:L:STD_ICON", 'ICON', 0, (void*) buf, a_info.size)) <= 0)
						error_check("node.WriteAttr(BEOS:L:STD_ICON) ", status);	}
				else error_check("node.ReadAttr(LABEL:L:ICON_BKP) ", status);
				delete buf;
				
				if ((status = node.GetAttrInfo("LABEL:M:ICON_BKP", &a_info)) == B_OK)
				{	buf = new char [a_info.size +1];
					if ((status = node.ReadAttr("LABEL:M:ICON_BKP", 'MICN', 0, (void*) buf, a_info.size)) > 0)
					{
						if ((status = node.WriteAttr("BEOS:M:STD_ICON", 'MICN', 0, (void*) buf, a_info.size)) <= 0)
							error_check("node.WriteAttr(BEOS:M:STD_ICON) ", status);
					}
					else error_check("node.ReadAttr(LABEL:M:ICON_BKP) ", status);
					delete buf;	}
					
				if ((status = node.RemoveAttr("LABEL:L:ICON_BKP")) != B_OK)					// Remove Backup
					error_check("node.RemoveAttr(LABEL:L:ICON_BKP) ", status);
				if ((status = node.RemoveAttr("LABEL:M:ICON_BKP")) != B_OK)
					error_check("node.RemoveAttr(LABEL:M:ICON_BKP) ", status);
			}
		}
		else															// No Label
		{
			printf("NO LABEL\n");
		}
		
		if (DoModify)	//	touch MODIFIED
		{
			BEntry	entry(&ref, true);
			entry.	SetModificationTime(time(NULL));
		}
		msg->RemoveData("refs");
	}
}

Label::Label	()
 : BApplication	("application/x-vnd.kirilla.label")	
{
	argv_received	=	false;
	refs_received	=	false;
}

Label::~Label	(void)	
{
	// void
}

void 
Label::ArgvReceived	(int32 argc, char **argv)
{
	argv_received	=	true;

	if(argc <= 1)	// no arguments
		return;

	BMessage msg (B_REFS_RECEIVED);

	for (int32 i = 1;  i < argc;  i++)
	{
		BEntry entry (argv[i]);

		entry_ref ref;
		entry.GetRef (& ref);
		
		if (entry.Exists())
			msg.AddRef ("refs", & ref);
		else
			printf("File not found: %s\n", argv[i]);
	}
	
	ProcessRefs(& msg);
}

void 
Label::RefsReceived	(BMessage * message)
{
	refs_received	=	true;

	ProcessRefs(message);
}

void
Label::ReadyToRun	(void)
{
	if (argv_received || refs_received)
	{
		PostMessage(B_QUIT_REQUESTED);
	}
	else
	{
		setWin * win = new setWin();
		win->Show();
	}
}

void 
Label::MessageReceived	(BMessage * a_message)
{
	switch(a_message->what)
	{
		//
				
		default:
				BApplication::MessageReceived(a_message);
				break;
	}
}

void 
Label::ProcessRefs(BMessage * msg)
{
//	init
	BPopUpMenu*		aPopUp	=	new BPopUpMenu("templates");
	BMenuItem*		aItem	=	NULL;
	IconMenuItem*	iItem	=	NULL;
	status_t  	status;
	bool	DoModify	=	true;
	bool	DoTint		=	true;
	
//	get settings
	BMessage settings;
	BPath path;
	if (find_directory(B_USER_SETTINGS_DIRECTORY, &path) == B_OK)
	{
		path.SetTo(path.Path(),"Mirilla");
		path.SetTo(path.Path(),"Label");
		BFile file;
		if ((status = file.SetTo(path.Path(),B_READ_ONLY)) == B_OK)
		{	settings.Unflatten(&file);	}
		else error_check("file.SetTo() ", status);
	}
	bool a;
	if (settings.FindBool("DoModify", &a) == B_OK)		DoModify = a;
	if (settings.FindBool("DoTint", &a) == B_OK)		DoTint = a;


//	Make popup
	aItem = new BMenuItem("no label", NULL);	aPopUp->AddItem(aItem);
	aPopUp->AddSeparatorItem();	

	BBitmap * color_bmp 	= new BBitmap (BRect(0,0,15,15), B_CMAP8, false);

	color_icon((uchar*) color_bmp->Bits(), color_bmp->BitsLength(), Orange);
	iItem = new IconMenuItem(settings.FindString("label1", (int32) 0), NULL,0,0, (BBitmap *) color_bmp);
	aPopUp->AddItem((BMenuItem*)iItem);	
	
	color_icon((uchar*) color_bmp->Bits(), color_bmp->BitsLength(), Red);
	iItem = new IconMenuItem(settings.FindString("label2", (int32) 0), NULL,0,0, (BBitmap *) color_bmp);
	aPopUp->AddItem((BMenuItem*)iItem);
	
	color_icon((uchar*) color_bmp->Bits(), color_bmp->BitsLength(), Lavender);
	iItem = new IconMenuItem(settings.FindString("label3", (int32) 0), NULL,0,0, (BBitmap *) color_bmp);
	aPopUp->AddItem((BMenuItem*)iItem);
	
	color_icon((uchar*) color_bmp->Bits(), color_bmp->BitsLength(), LightBlue);
	iItem = new IconMenuItem(settings.FindString("label4", (int32) 0), NULL,0,0, (BBitmap *) color_bmp);
	aPopUp->AddItem((BMenuItem*)iItem);
	
	color_icon((uchar*) color_bmp->Bits(), color_bmp->BitsLength(), Blue);
	iItem = new IconMenuItem(settings.FindString("label5", (int32) 0), NULL,0,0, (BBitmap *) color_bmp);
	aPopUp->AddItem((BMenuItem*)iItem);
	
	color_icon((uchar*) color_bmp->Bits(), color_bmp->BitsLength(), Green);
	iItem = new IconMenuItem(settings.FindString("label6", (int32) 0), NULL,0,0, (BBitmap *) color_bmp);
	aPopUp->AddItem((BMenuItem*)iItem);
	
	color_icon((uchar*) color_bmp->Bits(), color_bmp->BitsLength(), Black);
	iItem = new IconMenuItem(settings.FindString("label7", (int32) 0), NULL,0,0, (BBitmap *) color_bmp);
	aPopUp->AddItem((BMenuItem*)iItem);

	aPopUp->AddSeparatorItem();		
	aItem = new BMenuItem("settings" B_UTF8_ELLIPSIS, NULL);	aPopUp->AddItem(aItem);	// settings item
		
//	get Mouse
	BPoint mouse;
	mouseWin	*	win	=	new mouseWin(&mouse);
	delete	win;

//	make Popup
	BMenuItem * chosen = NULL;
	BString	label;

	chosen	=	aPopUp->Go(mouse, false, true);

//	handle popup
	if(chosen)	label = chosen->Label();
	else		{	delete aPopUp;	return;	}

	delete aPopUp;

//	Settings ?
	if (label == "settings" B_UTF8_ELLIPSIS)
	{
		argv_received	=	false;
		refs_received	=	false;
		// this will trigger window, in ReadyToRun()
		return;
	}
	
	
//	No Label
	if (label == "no label")
	{
		printf("<remove>\n");
		RemoveLabel	(msg, DoModify);
		printf("</remove>\n");
	}
	else
	{
		printf("<add>\n");
		AddLabel (msg, &label, DoTint, DoModify, &settings);
		printf("</add>\n");
	}
}

