#include <stdio.h>
#include <stdlib.h>

#include <Application.h>
#include <StringView.h>
#include <TextControl.h>
#include <ScrollView.h>
#include <Alert.h>
#include <Box.h>

#include <String.h>
#include <Beep.h>

#include <fs_info.h>
#include <fs_index.h>
#include <fs_attr.h>
#include <be/storage/File.h>
#include <be/storage/FindDirectory.h>
#include <be/storage/Path.h>
#include <be/storage/NodeMonitor.h>
#include <be/storage/VolumeRoster.h>
#include <be/storage/Volume.h>
#include <be/storage/Directory.h>
#include <be/storage/Query.h>

#include "VolumeIndexLooper.h"

VolumeIndexLooper::VolumeIndexLooper(dev_t	_device, VolumeIndexWindow * _window): BLooper(),
	device 		(_device),
	window		(_window),
	Running		(false)
{
	
}

VolumeIndexLooper::~VolumeIndexLooper()
{

}

void VolumeIndexLooper::MessageReceived(BMessage * msg)
{
	switch(msg->what)
	{
		case 'strt':	Start();		break;
				
		default:	BLooper::MessageReceived(msg);	break;
	}
}

void
VolumeIndexLooper::error_check(const char * msg, int32 status)
{
	printf(msg);
	const char *explanation = strerror(status);
	printf("(%s.)\n", explanation);
}

bool 
VolumeIndexLooper::IsRunning()	{	return Running;	}

void
VolumeIndexLooper::Start()
{
	printf("Looper::Start() -- IN\n");
	
	Running = true;

	// Reindex volume (device):
	// 
	// Query for name=* && META:label=*
	// read META:label
	// write META:label
	// ...
	
	// Query
	BQuery		query;
	BVolume		volume(device);
	entry_ref 	ref;
	status_t 	status;
	BNode 		node(&ref);
	attr_info	a_info;
	
	status = query	.SetVolume(&volume);				error_check("SetVolume", status);
	status = query	.SetPredicate("(name=*)");			error_check("SetPredicate", status);
	status = query	.Fetch();							error_check("Fetch", status);
	while((status = (query	.GetNextRef(&ref))) == B_OK)
	{
		node	.SetTo(&ref);
		if ((status = node.GetAttrInfo("META:label", &a_info)) == B_OK)
		{
			char * buf = new char [a_info.size +1];										// Backup Icon
			if ((status = node.ReadAttr("META:label", B_STRING_TYPE, 0, (void*) buf, a_info.size +1)) >= 0)
			{
				printf("file: %s\n", ref.name);
				//node.RemoveAttr("META:label"); //
		//snooze(4000000);
				if ((status = node.WriteAttr("META:label", B_STRING_TYPE, 0, (void*) buf, a_info.size +1)) < 0)
					error_check("node.WriteAttr(META:label) ", status);
				else
					printf("Attribute written.\n");
				
				// Hack
				BMessage update_msg(B_QUERY_UPDATE);
				int32	OpCode	=	B_ENTRY_CREATED;
				update_msg.AddInt32("opcode", OpCode);
				update_msg.AddInt32("device", device);
				window	->PostMessage(&update_msg);
				//***
			}
			else error_check("node.ReadAttr(META:label) ", status);
			delete buf;
		}
	}
	query.Clear();
	
	// Send 'fnsh' message to window
	// including -device-
	
	//snooze(5000000);	//
	
	BMessage msg('fnsh');
	msg.AddInt32("device", device);
	window->PostMessage(&msg);
			
	Running = false;
	
	printf("Looper::Start() -- OUT\n");
}
