#include <stdio.h>
#include <stdlib.h>

#include <Application.h>
#include <StringView.h>
#include <TextControl.h>
#include <ScrollView.h>
#include <Alert.h>
#include <Box.h>

#include <String.h>
#include <Beep.h>

#include <fs_info.h>
#include <fs_index.h>
#include <be/storage/File.h>
#include <be/storage/FindDirectory.h>
#include <be/storage/Path.h>
#include <be/storage/NodeMonitor.h>
#include <be/storage/VolumeRoster.h>
#include <be/storage/Volume.h>
#include <be/storage/Directory.h>
#include <be/storage/Query.h>

#include "VolumeIndexWindow.h"
#include "VolumeIndexLooper.h"
#include "BarberPole.h"


VolumeIndexWindow::VolumeIndexWindow()
	: BWindow(BRect(100,100,460,580), "Volumes", B_TITLED_WINDOW, B_NOT_H_RESIZABLE)
{
	SetPulseRate(50000);

	//	Find Settings
	BMessage	settings;
	BPath		path;
	status_t	status;
	if (find_directory(B_USER_SETTINGS_DIRECTORY, &path) == B_OK)
	{
		path.SetTo(path.Path(),"Mirilla");
		path.SetTo(path.Path(),"Label-Volumes");
		BFile file;
		if ((status = file.SetTo(path.Path(),B_READ_ONLY)) == B_OK)
		{	settings.Unflatten(&file);	}
		else error_check("file.SetTo() ", status);
	}
	BPoint loc;
	if (settings.FindPoint("winloc",&loc) == B_OK)	MoveTo(loc);

	//	Widget lists	
	vol_check_list	=	new	BList(5);
	
	// Making a BView
	label_view	=	new BView(BRect(0,0,360,19), "labels", B_FOLLOW_LEFT | B_FOLLOW_TOP, 0);
	item_view	=	new BView(BRect(0,20,345,440), "genres", B_FOLLOW_LEFT, B_NAVIGABLE);
	button_view	=	new BView(BRect(0,441,360,480), "buttons", B_FOLLOW_LEFT | B_FOLLOW_BOTTOM, B_NAVIGABLE);

	rgb_color grey	= {216, 216, 216, 0};	
	label_view		->	SetViewColor(grey);
	item_view		->	SetViewColor(grey);
	button_view		->	SetViewColor(grey);

	// Label view
	label_view		->	AddChild(new BStringView(BRect(10,-4,300,16), "Volume", 
								"Tick the volumes you want to be searchable.",
								 B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW));

	item_left	=	10;
	item_top	=	-25;
	item_right	=	110;
	item_bottom	=	0;
	
	// buttons
	help_button		=	new	BButton		(BRect(5,10,65,30), 	"help", "Help",			new BMessage('help'));
	reindex_button	=	new	BButton		(BRect(275,10,345,30), 	"reindex all", "Reindex All",	new BMessage('ridx'));
	button_view	->AddChild(help_button);
	button_view	->AddChild(reindex_button);
	
	my_scrollview	=	new BScrollView("scroll-view", item_view, B_FOLLOW_ALL_SIDES, 0, false, true, B_FANCY_BORDER);
	
	AddChild(my_scrollview);
	AddChild(button_view);
	AddChild(label_view);
	
	my_bar	=	item_view->ScrollBar(B_VERTICAL);
	if (my_bar != NULL)
	{	
		//my_bar->SetRange(0, item_bottom - 420);
		my_bar->SetRange(0, 420);
		my_bar->SetProportion((float) 420 / item_bottom);
		my_bar->SetSteps(10, 420); 
	}
	
	item_view		->ResizeTo(345,item_bottom);
	my_scrollview	->FrameResized(345, item_bottom);
	
	//	Getting window-location
	
	SetSizeLimits(360, 360, 60, item_bottom + 60);

	// Init lists
	vol_query_list	=	new	BList(5);	// list of BQuery
	vol_msgr_list	=	new	BList(5);	// list of BMessenger
	vol_string_list	=	new BList(5);	// list of BStringView
	vol_count_list	=	new BList(5);	// list of int64
	vol_info_list	=	new BList(5);	// list of fs_info *
	vol_looper_list	=	new BList(5);	// list of VolumeIndexLooper
	vol_barber_list	=	new BList(5);	// list of BarberPole
	vol_reindex_list = 	new BList(5);	// list of BButton
	vol_show_list = 	new BList(5);	// list of BButton
	
	// Starting Volume Roster	
	vol_roster		=	new BVolumeRoster();
	InitLinks();					// Getting current volumes
				
	if ((status = vol_roster->StartWatching(this)) != B_OK)
		error_check("\nCould not start BVolumeRoster... ", status);
}

VolumeIndexWindow::~VolumeIndexWindow()
{
	delete vol_roster;
	//...
}

void VolumeIndexWindow::MessageReceived(BMessage * msg)
{
	dev_t		device	=	0;
	fs_info		my_fs_info;

	switch(msg->what)
	{
	case 'help':	Help();				break;
	case 'volu':	UpdateIndices(true);	break;
	case 'ridx':	Reindex(msg);		break;
	case 'fnsh':	Finish(msg);		break;
	case 'show':	Show(msg);			break;
	
	case B_NODE_MONITOR:
			{
				int32	OpCode	=	0;
				if (msg->FindInt32("opcode", &OpCode) == B_OK)
				{
					if (OpCode == B_DEVICE_MOUNTED)
					{
						if (msg->FindInt32("new device", &device) == B_OK)
							if (fs_stat_dev(device, &my_fs_info) == 0)
								VolumeMounted	(&my_fs_info);
					}
					else if (OpCode == B_DEVICE_UNMOUNTED)
					{
						if (msg->FindInt32("device", &device) == B_OK)
							VolumeUnmounted	(device);
					}
				}
			}							break;
	case B_QUERY_UPDATE:
			{
				int32	OpCode	=	0;
				if (msg->FindInt32("opcode", &OpCode) == B_OK)
				{
					if (OpCode == B_ENTRY_CREATED)
					{
						if (msg->FindInt32("device", &device) == B_OK)
						{
							EntryCreated (device);
							printf("[+] %ld\n", device);
						}
					}
					else if (OpCode == B_ENTRY_REMOVED)
					{
						if (msg->FindInt32("device", &device) == B_OK)
						{
							EntryRemoved (device);
							printf("[-] %ld\n", device);
						}
					}
				}
			}							break;
	default:	msg->PrintToStream();
				BWindow::MessageReceived(msg);	break;
	}
}

void
VolumeIndexWindow::error_check(const char * msg, int32 status)
{
	printf(msg);
	const char *explanation = strerror(status);
	printf("(%s.)\n", explanation);
}

void
VolumeIndexWindow::Quit()
{
	 Hide();			// to make the exit-feeling faster
	fs_info	 * some_fs_info;
	while((some_fs_info 	=	(fs_info *) vol_info_list->ItemAt((int32) 0)) != NULL)
		VolumeUnmounted	(some_fs_info->dev);

	be_app->PostMessage(B_QUIT_REQUESTED);
	Save();
	BWindow::Quit();
}

void
VolumeIndexWindow::FrameResized	(float width, float height)
{
	my_bar->SetRange(0, item_bottom - (height-60));
	my_bar->SetProportion((float) (height-60) / item_bottom);
	my_bar->SetSteps(10, (height-60));
}

void 
VolumeIndexWindow::Zoom	(BPoint origin, float width, float height)
{
	if (height > (item_bottom + 60))
		height = (item_bottom + 60);
	
	ResizeTo(Bounds().right, height);
}

void 
VolumeIndexWindow::Update	()
{
	item_view		->ResizeTo(345,item_bottom);
	my_scrollview	->FrameResized(345, item_bottom);
	
	my_bar->SetRange(0, item_bottom - (Frame().Height()-60));
	my_bar->SetProportion((float) (Frame().Height()-60) / item_bottom);
	my_bar->SetSteps(25, (Frame().Height()-60));
	
	SetSizeLimits(360, 360, 60, item_bottom + 60);
	ResizeTo(360, item_bottom + 60);
}

void 
VolumeIndexWindow::Help()
{
	(new BAlert(0,	"BeOS can keep an index of your labeled files, "
					"to let you and your programs find them instantly.\n\n"

					"If a volume has an index for 'labeled files', "
					"each time you label a file, it's added to the index, "
					"and when you remove the label from a file, "
					"the file is removed from the index.\n\n"
					
					"An index is kept per volume, and it's up to you to "
					"decide which volumes you want to be indexed. "
					"In general, if you're using a volume to store labeled "
					"files, you'd want that volume to be indexed. "
					"Label's volume-preferences lets you add label-indices "
					"to your volumes by simply ticking their checkboxes.\n\n"

					"You can choose to remove the index from a volume, "
					"but keep in mind that you'll loose the ability to "
					"fast-find the labeled files of that volume. "
					"You will need to reindex the volume, which takes "
					"a few minutes for a large harddisk, to regain that functionality. "
					"Without the index, you can still find all files "
					"of a certain label, for example, but the find query "
					"will have to search through the entire disk.\n\n"
										
					"The index of a volume only contains files that were "
					"labeled after the creation of the index. Hint: reindex.\n\n",
		          	"Ok"))->Go();
}

void
VolumeIndexWindow::Save()
{
	BMessage settings;
	settings.AddPoint("winloc",Frame().LeftTop());

	BPath path;
	if (find_directory(B_USER_SETTINGS_DIRECTORY, &path) == B_OK)
	{
		BDirectory	set_dir(path.Path());
		BDirectory	m_dir;
		set_dir.CreateDirectory("Mirilla", &m_dir);
			
		path.SetTo(path.Path(),"Mirilla");
		path.SetTo(path.Path(),"Label-Volumes");

		BFile file;
		if (file.SetTo(path.Path(), B_WRITE_ONLY | B_ERASE_FILE | B_CREATE_FILE) == B_OK)
			settings.Flatten(&file);
	}
}

void
VolumeIndexWindow::InitLinks()
{
	int32		pos				=	0;
	dev_t		device			=	0;
	fs_info		my_fs_info;	
	while ((device = next_dev(&pos)) >= 0)
	{
		if (fs_stat_dev(device, &my_fs_info) == 0)
			VolumeMounted	(&my_fs_info);
	}
}

void
VolumeIndexWindow::VolumeMounted	(fs_info *	my_fs_info)
{
	printf("VolumeMounted\n");

	BVolume volume(my_fs_info->dev);
	if (volume.KnowsQuery() && volume.KnowsAttr())
	{
		printf("Volume: %s\n", my_fs_info->volume_name);
	
		item_top	+=	25;
		item_bottom	+=	25;
		
		// fs_info
		fs_info	 *	a	=	new	fs_info(*my_fs_info);
		vol_info_list->AddItem(a);
		
		// CheckBox
		BCheckBox	 *	check	= new BCheckBox(BRect(0, item_top, 100, item_bottom), "check",
												my_fs_info->volume_name, new BMessage('volu'));
		check		->SetValue	(HasIndexInstalled	(my_fs_info->dev));		
		vol_check_list	->AddItem(check);
		item_view	->AddChild(check);
		
		// int64
		int64	*	count	= new int64;
		*count	=	0;
		vol_count_list	->AddItem(count);
		
		// Query
		BQuery	*	query	=	new BQuery();
		vol_query_list	->AddItem(query);
		query	->	SetVolume(&volume);
		query	->	SetPredicate("'META:label'='*'");
			
		status_t status;
		entry_ref ref;
		query	->	Fetch();
		while((status = (query	->GetNextRef(&ref))) == B_OK)	(*count)++;
				
		// Messenger
		BMessenger * msgr	=	new BMessenger(NULL, (BLooper*) this, &status);
		vol_msgr_list	->AddItem(msgr);
		error_check("BMessenger(NULL, this, status)", status);
		
		query	->	Clear();
		query	->	SetVolume(&volume);
		query	->	SetPredicate("'META:label'='*'");
		error_check("query	->SetTarget(*msgr) ", query	->SetTarget(*msgr));
		query	->	Fetch();
		
		// StringView
		char * buf = new char [80];
		sprintf(buf, "%Ld labeled files", *count);
		BStringView	*	string	= new BStringView(BRect(110, (item_top-9), 230,(item_bottom-9)), "string",buf);
		vol_string_list	->AddItem(string);
		item_view	->AddChild(string);
		delete [] buf;
		
		// Show button	
		BMessage*		show_msg 	= new BMessage('show');
		show_msg->		AddInt32("device", my_fs_info->dev);
		BButton*		show_button =	new	BButton	(BRect(230, item_top, 280, (item_bottom-5)), "show", "Show", NULL);
		show_button->	SetMessage(show_msg);
		//show_button->	SetEnabled(false);
		vol_show_list->	AddItem(show_button);
		item_view->		AddChild(show_button);

		// Looper
		VolumeIndexLooper	*	looper	= new VolumeIndexLooper(my_fs_info->dev, this);
		vol_looper_list	->AddItem(looper);	
		looper		->Run();	// in VolumeMounted, since it should be called only once!

		// BarberPole		
		BarberPole * pole = new BarberPole(BRect(285, (item_top+5), 335, (item_bottom-5)), my_fs_info->volume_name,
													 B_FOLLOW_TOP | B_FOLLOW_LEFT, B_WILL_DRAW, FROM_RIGHT_TO_LEFT);
		vol_barber_list	->AddItem(pole);
		//item_view	->AddChild(pole);		// Don't add this -- wait for Reindex()
		//pole->Start();

		// Reindex button	
		BMessage *button_msg = new BMessage('ridx');
		button_msg->AddInt32("device", my_fs_info->dev);
		BButton * button =	new	BButton	(BRect(285, item_top, 335, (item_bottom-5)), "reindex", "Reindex", NULL);
		button	->SetMessage(button_msg);
		vol_reindex_list	->AddItem(button);
		item_view	->AddChild(button);
	}
	Update();
}

void
VolumeIndexWindow::VolumeUnmounted	(dev_t	device)
{
	printf("VolumeUnmounted -- IN\n");
	
	for (int32 i = vol_info_list->CountItems() -1;	i >= 0;	i--)
	{
		fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
		
		if (some_fs_info.dev == device)
		{
			item_top	-=	25;
			item_bottom	-=	25;
			
			// Remove fs_info
			fs_info	* a = (fs_info *) vol_info_list->RemoveItem(i);					// Remove from list
			delete a;																// delete 
		
			// Remove CheckBox
			BCheckBox * check 	=	(BCheckBox*) vol_check_list ->RemoveItem(i);	// Remove from list
			check			->RemoveSelf();											// Remove from view
			delete	check;															// delete 
			
			// Remove StringView
			BStringView * string 	=	(BStringView*) vol_string_list ->RemoveItem(i);	// Remove from list
			string			->RemoveSelf();												// Remove from view
			delete	string;																// delete 
			
			// Remove Query
			BQuery * query 	=	(BQuery*) vol_query_list ->RemoveItem(i);				// Remove from list
			query	->Clear();
			delete	query;																// delete 				
			
			// Remove Messenger
			BMessenger * msgr 	=	(BMessenger*) vol_msgr_list ->RemoveItem(i);		// Remove from list
			delete	msgr;																// delete 
			
			// Remove int64
			int64 * count 	=	(int64*) vol_count_list ->RemoveItem(i);				// Remove from list
			delete	count;																// delete 

			// Show button	
			BButton * s_button 	=	(BButton*) vol_show_list ->RemoveItem(i);			// Remove from list
			s_button			->RemoveSelf();												// Remove from view
			delete	s_button;																// delete 

			// Remove VolumeIndexLooper
			VolumeIndexLooper * looper 	=	(VolumeIndexLooper*) vol_looper_list ->RemoveItem(i);	// Remove from list
			looper	->Lock();
			looper	->Quit();																		// delete 

			// Remove BarberPole
			BarberPole * pole 	=	(BarberPole*) vol_barber_list ->RemoveItem(i);	// Remove from list
			if (pole	->IsRunning())
			{
				pole	->Stop();
				pole	->RemoveSelf();
			}
			delete	pole;															// delete 

			// Remove Reindex BButton
			BButton * r_button 	=	(BButton*) vol_reindex_list ->RemoveItem(i);			// Remove from list
			r_button			->RemoveSelf();												// Remove from view
			delete	r_button;																// delete 
				
			// DeleteVolume (i);
			// Move up list of widgets

			for (int32 j = i;  j < vol_check_list->CountItems();  j++)
			{
				((BCheckBox *)		vol_check_list	->ItemAt(j))->MoveBy(0,-25);
				((BStringView *)	vol_string_list	->ItemAt(j))->MoveBy(0,-25);
				((BarberPole *)		vol_barber_list	->ItemAt(j))->MoveBy(0,-25);
				((BButton *)		vol_reindex_list	->ItemAt(j))->MoveBy(0,-25);
			}
			
			Update();
			printf("Removing volume: %s\n", some_fs_info.volume_name);
			//return;
		}
	}

	printf("VolumeUnmounted -- OUT\n");
}

bool
VolumeIndexWindow::HasIndexInstalled	(dev_t	device)
{
	struct index_info idx_info;
	if 	((fs_stat_index(device, "META:label", &idx_info) != 0) || (idx_info.type != B_STRING_TYPE))
		return false;
	else
		return true;
}

status_t
VolumeIndexWindow::InstallIndex	(dev_t	device)
{
	if (fs_create_index(device, "META:label", B_STRING_TYPE, 0)	!= 0)
		return B_ERROR;
	else
		return B_OK;
}

status_t
VolumeIndexWindow::RemoveIndex	(dev_t	device)
{
	if (fs_remove_index(device, "META:label")	!= 0)
		return B_ERROR;
	else
	{
		for(int32 i = 0;	i < vol_info_list->CountItems();	i++)
		{
			fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
			
			if (some_fs_info.dev == device)
			{
				// int64
				int64	*	count;
				count 	=	(int64 *) vol_count_list->ItemAt(i);
				*count	=	0;
				
				// StringView
				char * buf = new char [80];
				sprintf(buf, "%Ld labeled files", *count);
				BStringView	*	string;
				string	= (BStringView *) vol_string_list->ItemAt(i);
				string	->SetText(buf);
				delete [] buf;
			}
		}
	}
	return B_OK;
}

void
VolumeIndexWindow::UpdateIndices(bool alert)
{
	printf("UpdateIndices()\n");
	
	for(int32 i = 0;	i < vol_info_list->CountItems();	i++)
	{
		fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
		BCheckBox * box			=	(BCheckBox*) vol_check_list ->ItemAt(i);
		
		if (HasIndexInstalled(some_fs_info.dev) != box->Value())
		{
			if (box->Value() == false)
			{
				if(alert)
				{
					BAlert * alert = new BAlert("RemoveIndex", 
						"Are you sure you want to remove the Label-attribute index "
						"from that volume?\n\n"
						
						"The index keeps track of which files have Label-"
						"attributes, and lets you find them instantly. "
						"If you remove the index, you can still search for "
						"all files of a certain label, but the filesystem "
						"query will need to search through the entire volume.\n\n"
						
						"If you remove the index, and, later on, would like to "
						"find all files with with a certain label, you need get back to "
						"Label's \"Make Labels Searchable\"-preferences, tick the volume's "
						"checkbox, and then press the \"Reindex Labeled Files\"-button."
						"This will go through all your indexed volumes, and "
						"make sure each file with a label is indexed, and ready to "
						"be found in Tracker's Find [by Attribute] or any other \"Live Query\".\n\n"
	
						"The index, of a volume, only contains attributes that were "
						"written after the creation of the index. Thus, the need for a Reindex function.\n\n"
						
						"It's recommended that you keep the index."
						,
						"Remove it", "Keep it", NULL, B_WIDTH_AS_USUAL, B_WARNING_ALERT);
	          
					int32 reply = alert->Go();
					
					if (reply == 1)
					{
						BCheckBox * check 	=	(BCheckBox*) vol_check_list ->ItemAt(i);
						check	->SetValue(true);
						
						return;
					}
				}
				RemoveIndex		(some_fs_info.dev);
			}
			else						InstallIndex	(some_fs_info.dev);

			return;
		}
	}
}

void
VolumeIndexWindow::EntryCreated	(dev_t	device)
{
	for(int32 i = 0;	i < vol_info_list->CountItems();	i++)
	{
		fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
		
		if (some_fs_info.dev == device)
		{
			// int64
			int64	*	count;
			count 	=	(int64 *) vol_count_list->ItemAt(i);
			(*count)++;
						
			// StringView
			char * buf = new char [80];
			sprintf(buf, "%Ld labeled files", *count);
			BStringView	*	string;
			string	= (BStringView *) vol_string_list->ItemAt(i);
			string	->SetText(buf);
			delete [] buf;
			return;
		}
	}
}

void
VolumeIndexWindow::EntryRemoved	(dev_t	device)
{
	for(int32 i = 0;	i < vol_info_list->CountItems();	i++)
	{
		fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
		
		if (some_fs_info.dev == device)
		{
			// int64
			int64	*	count;
			count 	=	(int64 *) vol_count_list->ItemAt(i);
			(*count)--;
						
			// StringView
			char * buf = new char [80];
			sprintf(buf, "%Ld labeled files", *count);
			BStringView	*	string;
			string	= (BStringView *) vol_string_list->ItemAt(i);
			string	->SetText(buf);
			delete [] buf;
			return;
		}
	}
}

void
VolumeIndexWindow::Reindex	(BMessage * msg)
{
	status_t	status	=	B_OK;
	dev_t		device	=	0;

	if ((status = (msg->FindInt32("device", &device))) == B_OK)
	{
		for(int32 i = 0;	i < vol_info_list->CountItems();	i++)
		{
			fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
			
			if (some_fs_info.dev == device)
			{
				VolumeIndexLooper * looper	=	(VolumeIndexLooper *) vol_looper_list->ItemAt(i);
				
				if (looper->IsRunning() == false)
				{
					BarberPole * pole	=	(BarberPole *)	vol_barber_list->ItemAt(i);
					BCheckBox * check	=	(BCheckBox *)	vol_check_list->ItemAt(i);
					
					if (pole->IsRunning() == false	&&	Lock())
					{
						check	->SetValue(false);
						RemoveIndex		(device);
						
						check	->SetValue(true);
						InstallIndex	(device);
						
						// int64
						int64	*	count;
						count 	=	(int64 *) vol_count_list->ItemAt(i);
						*count	=	0;
						
						// StringView
						char * buf = new char [80];
						sprintf(buf, "%Ld labeled files", *count);
						BStringView	*	string;
						string	= (BStringView *) vol_string_list->ItemAt(i);
						string	->SetText(buf);
						delete [] buf;
						
						BButton * button = (BButton *) vol_reindex_list->ItemAt(i);
						button->RemoveSelf();
						
						item_view	->AddChild(pole);
						pole		->Start();		
						looper		->PostMessage('strt');
						Unlock();
					}
				}
			}
		}
	}
	else
	{
		for(int32 i = 0;	i < vol_info_list->CountItems();	i++)
		{
			VolumeIndexLooper * looper	=	(VolumeIndexLooper *) vol_looper_list->ItemAt(i);
			
			if (looper->IsRunning() == false)
			{
				BarberPole * pole	=	(BarberPole *) vol_barber_list->ItemAt(i);
				BCheckBox * check	=	(BCheckBox *)	vol_check_list->ItemAt(i);
				
				if (pole->IsRunning() == false	&&	Lock())
				{
					fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
				
					check	->SetValue(false);
					RemoveIndex		(some_fs_info.dev);
						
					check	->SetValue(true);
					InstallIndex	(some_fs_info.dev);
					
					// int64
					int64	*	count;
					count 	=	(int64 *) vol_count_list->ItemAt(i);
					*count	=	0;
					
					// StringView
					char * buf = new char [80];
					sprintf(buf, "%Ld labeled files", *count);
					BStringView	*	string;
					string	= (BStringView *) vol_string_list->ItemAt(i);
					string	->SetText(buf);
					delete [] buf;
					
					BButton * button = (BButton *) vol_reindex_list->ItemAt(i);
					button->RemoveSelf();
					
					item_view	->AddChild(pole);
					pole		->Start();		
					looper		->PostMessage('strt');
					Unlock();
				}
			}
		}
	}
}

void 
VolumeIndexWindow::Finish	(BMessage * msg)
{
	status_t	status	=	B_OK;
	dev_t		device	=	0;

	if ((status = (msg->FindInt32("device", &device))) != B_OK)
		error_check("msg->FindInt32(device) ", status);
	else
	{
		for(int32 i = 0;	i < vol_info_list->CountItems();	i++)
		{
			fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
			
			if (some_fs_info.dev == device)
			{
				BarberPole * pole	=	(BarberPole *) vol_barber_list->ItemAt(i);
				pole	->Stop();				// Stanna BarberPole
				pole	->RemoveSelf();			// Ta bort BarberPole
				
				BButton * button = (BButton *) vol_reindex_list->ItemAt(i);
				item_view	->AddChild(button);
				
		// Restart query
				int64	newCount = 0;
				int64	* count 	=	(int64*) vol_count_list ->ItemAt(i);
				
				// stop live query
				BQuery * query 	=	(BQuery*) vol_query_list ->ItemAt(i);
				query	->Clear();
				
				// set Query
				BVolume	volume(device);
				query	->	SetVolume(&volume);
				query	->	SetPredicate("'META:label'='*'");
				
				// Query for all Labeled files on the volume
				entry_ref ref;
				query	->	Fetch();
				while((status = (query	->GetNextRef(&ref))) == B_OK)	newCount++;
								
				// Update counter and StringView
				*count = newCount;
				
				char * buf = new char [80];
				sprintf(buf, "%Ld labeled files", *count);
				BStringView	*	string = (BStringView *) vol_string_list->ItemAt(i);
				string	->SetText(buf);
				delete [] buf;
				
				// start live query
				BMessenger * msgr	=	(BMessenger *) vol_msgr_list->ItemAt(i);
				query	->	Clear();
				query	->	SetVolume(&volume);
				query	->	SetPredicate("'META:label'='*'");
				error_check("query	->SetTarget(*msgr) ", query	->SetTarget(*msgr));
				query	->	Fetch();
		//				
				
				return;
			}
		}
	}
}

void	
VolumeIndexWindow::Show	 	(BMessage * msg)
{
	status_t	status	=	B_OK;
	dev_t		device	=	0;

	if ((status = (msg->FindInt32("device", &device))) != B_OK)
		error_check("msg->FindInt32(device) ", status);
	else
	{
		for(int32 i = 0;	i < vol_info_list->CountItems();	i++)
		{
			fs_info	 some_fs_info 	=	*(fs_info *) vol_info_list->ItemAt(i);
			
			if (some_fs_info.dev == device)
			{
				(new BAlert(0,	"This feature hasn't been implemented yet.\n"
				"I need more info on the format of Query files.",
	          	"Sloppy!"))->Go();
				return;
			}
		}
	}
}
